/* Copyright (C) 2003 Stephane Thiell
 *
 * This file is part of pxyservd (from pxys)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
 *
 */
#define RCSID "$Id: glineq.c,v 1.2 2004/01/15 01:46:50 mbuna Exp $"

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include "glineq.h"

#include <assert.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <netinet/in.h>

#if 0
#include <string.h>
#include <stdio.h>
#endif

#define Q_SPACE_INIT_SIZE 32
#define Q_SPACE_GROW_SIZE 32

struct g_data
  {
  struct in_addr addr;
  int hitcnt;
  char reason[248];
  };

struct g_elem
  {
  int next_idx;
  struct g_data data;
  };

struct g_space
  {
  int sblk, sblk_end; /* index */
  int nelem;
  int size;
  int maxelems;
  struct g_elem *space;
  };

static struct g_space qspace;
static int qhead_idx = -1, qtail_idx;
static int qsize;

static void
qspace_grow()
  {
  qspace.sblk_end = qspace.size;
  qspace.size += Q_SPACE_GROW_SIZE;
  qspace.space = (struct g_elem *)realloc(qspace.space, qspace.size
                                          * sizeof(struct g_elem));
  }

void
glineq_init(int max_queue_elems)
  {
  qspace.nelem = 0;
  qspace.size = Q_SPACE_INIT_SIZE;
  qspace.maxelems = max_queue_elems;
  qspace.space = (struct g_elem *)malloc(qspace.size * sizeof(struct g_elem));
  qspace.sblk = qspace.sblk_end = 0;
  }

void
glineq_finalize()
  {
  free(qspace.space);
  }

int
glineq_empty()
  {
  return qsize == 0;
  }

int
glineq_push(const struct g_args *args)
  {
  struct g_elem *e;
  int idx;
  
  if (qspace.maxelems && (qspace.nelem >= qspace.maxelems))
    return -1; /* overflow */
  
  if (qspace.nelem >= qspace.size)
    qspace_grow();
  assert(qspace.nelem < qspace.size);
  
  if (qspace.sblk_end >= qspace.size)
    qspace.sblk_end = 0;
  
  idx = qspace.sblk_end++;
  e = &qspace.space[idx];
  qspace.nelem++;
  e->next_idx = -1;
  
  e->data.addr = args->addr;
  e->data.hitcnt = args->hitcnt;
  snprintf(e->data.reason, sizeof(e->data.reason), "%s", args->reason);
  
  if (qhead_idx == -1)
    qhead_idx = idx;
  else
    qspace.space[qtail_idx].next_idx = idx;
  
  qtail_idx = idx;
  qsize++;
  
  return 0;
  }

const struct g_args *
glineq_first()
  {
  static struct g_args args;
  struct g_args *argsp = NULL;
  struct g_data *d;
  
  assert(qhead_idx >= 0);
  
  if (qsize > 0)
    {
    d = &qspace.space[qhead_idx].data;
        
    args.addr = d->addr;
    args.hitcnt = d->hitcnt;
    args.reason = d->reason;
    argsp = &args;
    }
  return argsp;
  }

void
glineq_pop()
  {
  assert(qsize > 0);
  assert(qhead_idx >= 0);
  qhead_idx = qspace.space[qhead_idx].next_idx;
  qsize--;

  assert(qspace.nelem > 0);

  if (qspace.sblk >= qspace.size)
    qspace.sblk = 0;
  qspace.sblk++;
  qspace.nelem--;
  }

#if 0
int
main()
  {
  char reasonbuf[256];
  int i;
  struct in_addr ip;
  struct g_args arg;
  
  glineq_init(10);
    
  for (i = 0; i < 900; i++)
    {
    arg.addr.s_addr = i;
    arg.reason = reasonbuf;
    snprintf(reasonbuf, sizeof(reasonbuf), "Reason %d", i);
    glineq_push(&arg);
    }
    
  while (qhead_idx >= 0)
    {
    printf("gline '%s' '%s'\n", inet_ntoa(qspace.space[qhead_idx].data.addr),
           qspace.space[qhead_idx].data.reason);
    glineq_pop();
    }

  return 0;
  }
#endif
